; ---------------------------------------------------------------------------
;  Copyright (C) 2003-2004 Dallas Semiconductor Corporation, All Rights Reserved.
; 
;  Permission is hereby granted, free of charge, to any person obtaining a
;  copy of this software and associated documentation files (the "Software"),
;  to deal in the Software without restriction, including without limitation
;  the rights to use, copy, modify, merge, publish, distribute, sublicense,
;  and/or sell copies of the Software, and to permit persons to whom the
;  Software is furnished to do so, subject to the following conditions:
; 
;  The above copyright notice and this permission notice shall be included
;  in all copies or substantial portions of the Software.
; 
;  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
;  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
;  MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
;  IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
;  OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
;  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
;  OTHER DEALINGS IN THE SOFTWARE.
; 
;  Except as contained in this notice, the name of Dallas Semiconductor
;  shall not be used except as stated in the Dallas Semiconductor
;  Branding Policy.
; ---------------------------------------------------------------------------
;
;  This file contains code declarations for exported ROM functions.
;  Only include this file if you are actually calling one of these ROM 
;  exported functions.
;
; ---------------------------------------------------------------------------
;

;
; ROM EXPORT TABLE FUNCTIONS (denoted with ROMXT)
;

; UTILITY functions 
EXTERN ECODE(ROMXT_CRC16)
EXTERN ECODE(ROMXT_MEM_CLEAR_16)
EXTERN ECODE(ROMXT_MEM_COPY_16)
EXTERN ECODE(ROMXT_MEM_COMPARE)
EXTERN ECODE(ROMXT_ADD_DPTR_0_16) ; not implemented for C compiler
EXTERN ECODE(ROMXT_ADD_DPTR_1_16) ; not implemented for C compiler
EXTERN ECODE(ROMXT_SUB_DPTR_0_16) ; not implemented for C compiler
EXTERN ECODE(ROMXT_SUB_DPTR_1_16) ; not implemented for C compiler
EXTERN ECODE(ROMXT_GETPSEUDORANDOM)

; MEMORY MGR
EXTERN ECODE(ROMXT_KERNELMALLOC) ; not exposed
EXTERN ECODE(ROMXT_KERNELFREE) ; not exposed
EXTERN ECODE(ROMXT_MM_MALLOC) ; exposed as redirected function
EXTERN ECODE(ROMXT_MM_MALLOC_DIRTY) ; exposed as redirected function
EXTERN ECODE(ROMXT_MM_FREE) ; exposed as redirected function
EXTERN ECODE(ROMXT_MM_DEREF) ; exposed as redirected function
EXTERN ECODE(ROMXT_GETFREERAM) ; exposed as redirected function

; SOCKET functions
EXTERN ECODE(ROMXT_ROM_SOCKET)
EXTERN ECODE(ROMXT_ROM_CLOSESOCKET)
EXTERN ECODE(ROMXT_ROM_SENDTO)
EXTERN ECODE(ROMXT_ROM_RECVFROM)
EXTERN ECODE(ROMXT_ROM_CONNECT)
EXTERN ECODE(ROMXT_ROM_BIND)
EXTERN ECODE(ROMXT_ROM_LISTEN)
EXTERN ECODE(ROMXT_ROM_ACCEPT)
EXTERN ECODE(ROMXT_ROM_RECV)
EXTERN ECODE(ROMXT_ROM_SEND)
EXTERN ECODE(ROMXT_ROM_GETSOCKOPT)
EXTERN ECODE(ROMXT_ROM_SETSOCKOPT)
EXTERN ECODE(ROMXT_ROM_GETSOCKNAME)
EXTERN ECODE(ROMXT_ROM_GETPEERNAME)
EXTERN ECODE(ROMXT_ROM_CLEANUP)
EXTERN ECODE(ROMXT_ROM_AVAIL)
EXTERN ECODE(ROMXT_ROM_JOIN)
EXTERN ECODE(ROMXT_ROM_LEAVE)
EXTERN ECODE(ROMXT_ROM_PING)
EXTERN ECODE(ROMXT_ROM_GETNETWORKPARAMS)
EXTERN ECODE(ROMXT_ROM_SETNETWORKPARAMS)
EXTERN ECODE(ROMXT_ROM_GETIPV6PARAMS)
EXTERN ECODE(ROMXT_ROM_GETETHERNETSTATUS)
EXTERN ECODE(ROMXT_ROM_GETTFTPSERVER)
EXTERN ECODE(ROMXT_ROM_SETTFTPSERVER)
EXTERN ECODE(ROMXT_ETH_PROCESSINTERRUPT) ; not implemented for C compiler
EXTERN ECODE(ROMXT_ARP_GENERATER) ; not implemented for C compiler
EXTERN ECODE(ROMXT_NET_ETH0_MAC_ID)

; DHCP functions
EXTERN ECODE(ROMXT_DHCP_INIT)
EXTERN ECODE(ROMXT_DHCP_SETUP) ; not implemented for C compiler
EXTERN ECODE(ROMXT_DHCP_STARTUP) ; not implemented for C compiler
EXTERN ECODE(ROMXT_DHCP_RUN) ; not implemented for C compiler
EXTERN ECODE(ROMXT_DHCP_STATUS)
EXTERN ECODE(ROMXT_DHCP_STOP)
EXTERN ECODE(ROMXT_DHCPNOTIFY) ; empty redirect stub, not implemented

; TFTP functions
EXTERN ECODE(ROMXT_TFTP_INIT)
EXTERN ECODE(ROMXT_TFTP_FIRST)
EXTERN ECODE(ROMXT_TFTP_NEXT)
EXTERN ECODE(ROMXT_TFTP_MSG)

; SCHEDULER functions
EXTERN ECODE(ROMXT_TASK_GENESIS)
EXTERN ECODE(ROMXT_TASK_GETCURRENT)
EXTERN ECODE(ROMXT_TASK_GETPRIORITY)
EXTERN ECODE(ROMXT_TASK_SETPRIORITY)
EXTERN ECODE(ROMXT_TASK_FORK)
EXTERN ECODE(ROMXT_TASK_KILL)
EXTERN ECODE(ROMXT_TASK_SUSPEND)
EXTERN ECODE(ROMXT_TASK_SLEEP)
EXTERN ECODE(ROMXT_TASK_SIGNAL)
EXTERN ECODE(ROMXT_ROM_TASK_SWITCH_IN) ; empty redirect stub, not implemented
EXTERN ECODE(ROMXT_ROM_TASK_SWITCH_OUT) ; empty redirect stub, not implemented
EXTERN ECODE(ROMXT_ENTERCRITSECTION)
EXTERN ECODE(ROMXT_LEAVECRITSECTION)

; INIT functions
EXTERN ECODE(ROMXT_ROM_INIT)
EXTERN ECODE(ROMXT_ROM_COPYIVT)
EXTERN ECODE(ROMXT_ROM_REDIRECT_INIT)
EXTERN ECODE(ROMXT_MM_INIT)
EXTERN ECODE(ROMXT_KM_INIT)
EXTERN ECODE(ROMXT_OW_INIT)
EXTERN ECODE(ROMXT_NETWORK_INIT)
EXTERN ECODE(ROMXT_ETH_INIT)
EXTERN ECODE(ROMXT_INIT_SOCKETS)
EXTERN ECODE(ROMXT_TICK_INIT)

; Timer Interrupt vectors
EXTERN ECODE(ROMXT_WOS_TICK) ; not implemented for C compiler
EXTERN ECODE(ROMXT_BLOB) ; not implemented for C compiler

; Maintenance functions
EXTERN ECODE(ROMXT_WOS_IOPOLL) ; not implemented for C compiler
EXTERN ECODE(ROMXT_IP_PROCESSRECEIV) ; not implemented for C compiler
EXTERN ECODE(ROMXT_IP_PROCESSOUTPUT) ; not implemented for C compiler
EXTERN ECODE(ROMXT_TCP_RETRYTOP) ; not implemented for C compiler
EXTERN ECODE(ROMXT_ETH_PROCESSOUTPUT) ; not implemented for C compiler
EXTERN ECODE(ROMXT_IGMP_GROUPMAINTAINENCE) ; not implemented for C compiler
EXTERN ECODE(ROMXT_IP6_PROCESSRECEIV) ; not implemented for C compiler
EXTERN ECODE(ROMXT_IP6_PROCESSOUTPUT) ; not implemented for C compiler
EXTERN ECODE(ROMXT_PARAMBUFFER) ; not implemented for C compiler
EXTERN ECODE(ROMXT_RAM_TOP) ; not implemented for C compiler
EXTERN ECODE(ROMXT_BOOT_MEMBEGIN) ; not implemented for C compiler
EXTERN ECODE(ROMXT_BOOT_MEMEND) ; not implemented for C compiler

; 1-Wire
EXTERN ECODE(ROMXT_OWM_FIRST)
EXTERN ECODE(ROMXT_OWM_NEXT)
EXTERN ECODE(ROMXT_OWM_RESET)
EXTERN ECODE(ROMXT_OWM_BYTE)
EXTERN ECODE(ROMXT_OWM_SEARCH) ; not implemented for C compiler
EXTERN ECODE(ROMXT_OW_ROMID)

; Misc, extras, late additions
EXTERN ECODE(ROMXT_AUTOBAUD)
EXTERN ECODE(ROMXT_TFTP_CLOSE)

; 1.1 Additions
EXTERN ECODE(ROMXT_NETBOOT)
EXTERN ECODE(ROMXT_TASK_SWITCHER)
EXTERN ECODE(ROMXT_TICK_CALCULATERELOAD)
EXTERN ECODE(ROMXT_OWM_PROBECLOCK)
EXTERN ECODE(ROMXT_OWM_CALCULATEDIVISOR)
EXTERN ECODE(ROMXT_INFO_SENDSTRING)
EXTERN ECODE(ROMXT_INFO_SENDHEXBYTE)
EXTERN ECODE(ROMXT_INFO_CONVHEX)
EXTERN ECODE(ROMXT_INFO_SENDCRLF)
EXTERN ECODE(ROMXT_COPYRIGHT)
EXTERN ECODE(ROMXT_ALLRIGHTSRESERVED)
EXTERN ECODE(ROMXT_PSEUDORANDOMSEED)
EXTERN ECODE(ROMXT_FLASHSECTORERASE)
EXTERN ECODE(ROMXT_FLASHPROGRAMBYTE)
EXTERN ECODE(ROMXT_ARP_GENERATER_NOCHECK)
EXTERN ECODE(ROMXT_ARP_CHECKCACHE)
EXTERN ECODE(ROMXT_ROM_SUBNETTOPREFIX)
EXTERN ECODE(ROMXT_ROM_UNBIND)
EXTERN ECODE(ROMXT_MATH_MUL1024)
EXTERN ECODE(ROMXT_MATH_DIV2)
EXTERN ECODE(ROMXT_MATH_DIV1024)
EXTERN ECODE(ROMXT_MATH_LONGDIV1024)
EXTERN ECODE(ROMXT_TASK_SUSPEND_NC)
EXTERN ECODE(ROMXT_TASK_SLEEP_NC)
EXTERN ECODE(ROMXT_UDP_TESTRECEIVE)
EXTERN ECODE(ROMXT_ETH_READMII)
EXTERN ECODE(ROMXT_ETH_WRITEMII)
EXTERN ECODE(ROMXT_ETH_READCSR)
EXTERN ECODE(ROMXT_ETH_WRITECSR)
EXTERN ECODE(ROMXT_IP_CHECKHEADER)
EXTERN ECODE(ROMXT_IP_PACKETRECEIVED)
